# frozen_string_literal: true

module Hermes
  class LogJob < ActiveJob::Base
    queue_as :x_hermes_log

    QUEUE_NAME = 'redis_hermes_logs_queue'
    PROCESS_QUEUE_NAME = 'redis_hermes_logs_queue_process'
    QUEUE_MAX_SIZE = 100
    
    def perform
      bulk_write = []

      loop do
        break unless attributes_json = $redis.rpoplpush(QUEUE_NAME, PROCESS_QUEUE_NAME)
        bulk_write.push({ insert_one: JSON.parse(attributes_json)})
        $redis.lrem(PROCESS_QUEUE_NAME, 0, attributes_json) 
      end 

      Log.collection.bulk_write(bulk_write)
    end

    def self.add_to_queue(attributes, options = {})
      return unless attributes

      force_perform_now = options[:force_perform_now]
      attributes[:level] = 'info' unless attributes[:level].present?
      attributes[:domain] = 'api' unless attributes[:domain].present?

      $redis.lpush(QUEUE_NAME, attributes.to_json)

      if queue_size >= queue_max_size
        force_perform_now ? perform_now : perform_later 
      end
    end

    def self.queue_size
      $redis.llen(QUEUE_NAME)
    end 

    def self.queue_max_size
      QUEUE_MAX_SIZE
    end 
  end
end
